%%%
%%% 2023-12-7. Nova de Hi
%%% 2023-12-9. bigindent option.
%%% 2023-12-11. verse
%%% 2023-12-13. comma rlap and others
%%% 2023-12-14. main latin to TeX Gyre Chorus
%%% 2023-12-15. fine tuning and Ewords
%%% 2023-12-18. stretchshrink fine tuning.
%%% 2023-12-19. \dialog bug fix concerning period.
%%% 2023-12-21. shortperiod
%%%
\ProvidesClass{wongoji}[2023/12/21 version 0.12 WONGOJI (manuscript paper) class]

\RequirePackage{l3keys2e}

\ExplSyntaxOn

\keys_define:nn { wongojipkg }
{
	200	.bool_set_inverse:N = \opt_four_bool,
	400	.bool_set:N = \opt_four_bool,
	bigindent 	.bool_set:N = \opt_bigindent_bool,
	shortperiod	.bool_set:N = \opt_shortperiod_bool,
}

\keys_set:nn { wongojipkg }
{
	200 = true,
	400 = false,
	bigindent = false,
	shortperiod = false,
}

\ProcessKeysOptions { wongojipkg } 

\ExplSyntaxOff

\LoadClass[12pt,oldfontcommands]{oblivoir}

\newif\iftwohundred
\newif\ifbigIndent

\newcommand*\Xquoteindentbig {\bigIndenttrue}
\newcommand*\Xquoteindentsmall {\bigIndentfalse}

\ExplSyntaxOn

\bool_if:NTF \opt_bigindent_bool
{
	\bigIndenttrue
}
{
	\bigIndentfalse
}

\RequirePackage{fapapersize}
\bool_if:NTF \opt_four_bool
{
	\usefapapersize{210mm,297mm,20mm,\dimexpr190mm-40em+2pt\relax,25mm,80mm}
	\tl_set:Nn \bggraphic { wongojiwhite400 }
	\twohundredfalse
}
{
	\usefapapersize{210mm,148mm,20mm,\dimexpr190mm-40em+2pt\relax,20mm,30mm}
	\tl_set:Nn \bggraphic { wongojiwhite }
	\twohundredtrue
}

\ExplSyntaxOff

\RequirePackage{etoolbox}
\RequirePackage{xcolor,ninecolors}
\RequirePackage{graphicx}
\graphicspath{{./}{./images}}
\RequirePackage{tzplot}
\usetikzlibrary{tikzmark}
\RequirePackage{ragged2e}
\RequirePackage{adjustbox}
\RequirePackage[normalem]{ulem}
\RequirePackage{oblivoir-misc}
\RequirePackage{amssymb}

\parindent=0pt

\newlength\pagematchdimen
\setlength\pagematchdimen{\dimexpr\textheight+33pt\relax}
\newlength\leftydimen
\setlength\leftydimen{2pt}
\copypagestyle{wongoji}{empty}
\makeoddfoot{wongoji}{%
    \begin{tikzpicture}[remember picture,overlay]
    \begin{pgfonlayer}{above}
%    \foreach \x in {0, 2.3, ..., 23}
%    	{ \tzline [red6]<-\leftydimen,\pagematchdimen> (0,-\x em)(40em,-\x em) }
%    \foreach \x in {2, 4.3, ..., 22.7}
%    	{ \tzline [red6]<-\leftydimen,\pagematchdimen> (0,-\x em)(40em,-\x em) }
%    \tzrectangle[red6,very thick]<-\leftydimen,\pagematchdimen>(0,0)(40em,-22.7em)
%    \foreach \i in {0,2,...,40}
%    	{
%    		\foreach \j in {0,2.3,...,22.7}
%    			{ \tzline+ [red6] <-\leftydimen,\pagematchdimen> (\i em,-\j em)(0,-2em) } 
%    	}
	\tznode[anchor=north west]<-\leftydimen-5pt,\pagematchdimen+4.5pt>(0,0){\includegraphics{\bggraphic}}
    \tznode[anchor=south east,red4]<-\leftydimen,\pagematchdimen>(40em,-1pt){\color{red4}\uline{\fontspec{Zapfino.ttf}\footnotesize No. \color{magenta3}\thepage}}
    \end{pgfonlayer}
    \end{tikzpicture}
}{}{}

\setkomainfont(UnPenheulim.ttf)[interhchar=0.08em,WordSpace={2,0,0}]

\AtBeginDocument{
	\fontspec{TeX Gyre Chorus}
	\hangulfontspec{UnPenheulim.ttf}[interhchar=0.08em,WordSpace={2,0,0}]
	\hanjafontspec{Noto Serif CJK KR}[Scale=.95,WordSpace={2,0,0}]
	\def\XKinterhanjabreak{\hspace{2.1pt}}
	\def\XeKo@stretchshrink{plus .5pt minus 2pt\relax}
	\everyhangul{\allowbreak\fcolorbox{white}{white}{\adjustbox{width=1em}{#1}}}
	\everyhanja{\allowbreak\fcolorbox{white}{white}{\adjustbox{width=1em}{#1}}}
	% \hangulmarks
	\fontsize{1.33em}{1.72em}\selectfont
	\RaggedRight\spaceskip=1.565em plus 0em minus 0em\xspaceskip=2.77em
	\clubpenalty=0\widowpenalty=0\hyphenpenalty=0
	\pagestyle{wongoji}
	\everypar{\null\Xindent}
}

\newcommand*\Xindent{\hphantom{각}}
\newcommand*\Xnoindent{\hspace*{-1.5em}}
\newcommand*\Xbigskip{\vphantom{각}\par}
%\newcommand*\comma{\:\allowbreak,\hspace{1.03em}}
\newcommand*\Xcomma{\nobreak\rlap{\;,}\allowbreak\hspace{1.5em}}
\let\comma\Xcomma
\newcommand*\Xprd{\rlap{\;。}\allowbreak\hskip 3.05em plus .5pt minus 2pt}
\let\Xaltprd\Xprd
\newcommand*\Xprdsym{。}
\newcommand*\commasym{\char"002C}
\newcommand*\Xb[1]{{\fboxsep0pt\fcolorbox{white}{white}{\makebox[1.48em][c]{#1}}}}
\let\X\Xb

\newcommand*\Xnewline{\newline \Xnoindent}
\newcommand*\Xellipsis{\adjustbox{raise=2pt}{\hangulfontspec{Noto Serif CJK KR}\kern-1pt\Xb{…}\Xb{…}}}
\newcommand*\Xdots{\Xb{…}}

\RequirePackage{kslinematters}
\newlinecommand\dialog{\DialogFn{#1}}

\ExplSyntaxOn

\NewDocumentCommand \Xbb { o m }
{
	\IfValueTF { #1 }
	{
		\int_set:Nn \l_tmpa_int { #1 }
	}
	{
		\int_set:Nn \l_tmpa_int { 2 }
	}
	
	\dim_set:Nn \l_tmpa_dim { \l_tmpa_int em }
	\dim_set:Nn \l_tmpa_dim { 1.48 \l_tmpa_dim }
	
	\makebox[\l_tmpa_dim ][s]{ \spaceskip=3pt plus 2pt minus 2pt #2 }
}


\NewDocumentCommand \Xlap { s m }
{
	\IfBooleanTF { #1 }
	{
		\llap{\Xb{#2}}
	}
	{
		\rlap{\Xb{#2}}
	}
}

\dim_new:N \afterPrd_dim 
\dim_set:Nn \afterPrd_dim { 3.05 em }

\NewDocumentCommand \DialogFn { m }
{
	\tl_set:Nn \l_tmpa_tl { #1 }
	\tl_reverse:N \l_tmpa_tl
	\tl_set:Nx \l_tmpb_tl { \tl_head:N \l_tmpa_tl }
	\tl_set:Nx \l_tmpc_tl { \tl_tail:N \l_tmpa_tl }
	\tl_reverse:N \l_tmpc_tl
	
	\group_begin:
	\tl_set_eq:NN \Xprd \Xprdsym
	\begin{adjustwidth}{1.36em}{0pt}
	\ifbigIndent\Xindent\fi
	\,\Xb{“} 
	{
		\spaceskip=1.565em plus 0em minus 0em
		\renewcommand*\Xprd{\rlap{\;。}\kern \afterPrd_dim\allowbreak}
		\tl_use:N \l_tmpc_tl
	}
	\str_case_e:nnTF { \l_tmpb_tl }
	{
		{ ? } { } { ! } { }
	}
	{
		\kern2pt \Xb { \tl_use:N \l_tmpb_tl } \Xb {”}
	}
	{
		\Xb{\kern2pt\tl_use:N \l_tmpb_tl\, ”}
	}
	\end{adjustwidth}
	\vspace{4pt}
	\group_end:
}

\NewDocumentCommand \hjparen { O{()} m }
{
	\tl_set:Nn \l_tmpl_tl { #1 }
	\tl_set:Nx \l_tmpm_tl { \tl_head:N \l_tmpl_tl }
	\tl_set:Nx \l_tmpn_tl { \tl_tail:N \l_tmpl_tl }
	
	\kern.1em\Xb{\l_tmpm_tl} #2 \nobreak\rlap{\X{\l_tmpn_tl}}\allowbreak\hspace{1.5em}
}

\RenewDocumentEnvironment{verse}{}
{
	\group_begin:
	\parindent=1.5em
	\obeylines
}
{
	\group_end:
}

\NewDocumentCommand \ExplDash {}
{
	\adjustbox{raise=2.8pt}{\hbox to 3em{\hss\rule{2.3em}{.6pt}\hss}}
}

\RenewDocumentCommand \@ {}
{ \hspace{0.5em} }

\newcounter{indentparenum}

\NewDocumentEnvironment {indentPar } { O{1} }
{
	\str_case:nn { #1 }
	{
		{ 1 } { \dim_set:Nn \l_tmpa_dim { 1.35em} }
		{ 2 } { \dim_set:Nn \l_tmpa_dim { 3em} }
		{ 3 } { \dim_set:Nn \l_tmpa_dim { 4.5em } }
	}
	
	\par\begingroup
	\setcounter{indentparenum}{0}
	\begin{adjustwidth}{\l_tmpa_dim }{0pt}
}
{
	\end{adjustwidth}
	\endgroup\par
	\vspace{4pt}
}

\tl_new:N \l_subtitle_tl

\newcommand*\subtitle[1]{ \tl_set:Nn \l_subtitle_tl { #1 } }

\bool_new:N \g_usertitle_bool 

\RenewDocumentCommand \title { s m }
{
	\IfBooleanTF { #1 }
	{
		\gdef\@title{#2}
		\bool_gset_false:N \g_usertitle_bool
	}
	{
		\gdef\@title{#2}
		\bool_gset_true:N \g_usertitle_bool
	}
}

\RenewDocumentCommand \maketitle {}
{
	\Xnoindent\Xbigskip
	\bool_if:NTF \g_usertitle_bool
	{
		\exp_args:Nnx \regex_split:nnN { \s } { \@title } \l_tmpa_seq
		\tl_set:Nx \l_tmpa_tl { \@title }
		\int_set:Nn \l_tmpa_int { \tl_count:N \l_tmpa_tl + \seq_count:N \l_tmpa_seq -1 }
		\int_if_odd:nT { \l_tmpa_int } { \tl_put_left:Nn \l_tmpa_tl { \Xindent } }
		\centerline { \l_tmpa_tl }
	}
	{
		\centerline { \@title }
	}
	\tl_if_empty:NF \l_subtitle_tl
	\centerline{\l_subtitle_tl}
	\Xbigskip
	\rightline{\@author\Xindent}
	\Xbigskip
}

\RenewDocumentCommand \section { s m }
{
	\refstepcounter{section}
    \addcontentsline{toc}{section}{
	    \texorpdfstring{\string\spaceskip=1.565em plus 0em minus 0em\string\xspaceskip=2.77em}{}
        \protect\numberline{\csname thesection\endcsname}#2
    }
	\IfBooleanF { #1 }
	{
		\Xnoindent\Xbigskip
	}
	\begingroup
	\Xnoindent 제\Xb{\thesection}절~{}#2
	\par
	\endgroup
	\Xbigskip
}

\RenewDocumentCommand \subsection { s m }
{
	\refstepcounter{subsection}
    \addcontentsline{toc}{subsection}{
	    \texorpdfstring{\string\spaceskip=1.565em plus 0em minus 0em\string\xspaceskip=2.77em}{}
        \protect\numberline{\csname thesubsection\endcsname}#2
    }
	\IfBooleanF { #1 }
	{
		\Xbigskip
	}
	\begingroup
	\Xnoindent \Xbb{\thesubsection}#2
	\par
	\endgroup
%	\Xbigskip
}

\newcommand*\LQ{\X“}
\NewDocumentCommand \RQ { o }
{
	\IfNoValueTF { #1 }
	{ \X{”} }
	{
		\str_case:nnTF { #1 }
		{
			{ . } { \X{\Xprdsym ”} }
			{ , } { \X{\commasym ”} }
		}
		{}
		{ \Xb{#1}\Xb{”} }
	}
}

\newcommand*\Lq{\X‘}
\NewDocumentCommand \Rq { o }
{
	\IfNoValueTF { #1 }
	{ \X{’} }
	{
		\str_case:nnF { #1 }
		{
			{ . } { \X{\Xprdsym ’} }
			{ , } { \X{\commasym ’} }
		}
		{ \Xb{#1}\Xb{’} }
	}
}


\def\dotemphraise{.9em }
\def\dotemphchar{\tikz\tzdot*(0,5pt)(2.5pt);}

\RenewDocumentCommand \emph { m }
{
	\regex_split:nnN { \s } { #1 } \l_tmpa_seq
	\seq_map_indexed_inline:Nn \l_tmpa_seq
	{
		\dotemph{\em ##2}
		\int_compare:nT { ##1 < \seq_count:N \l_tmpa_seq }
		{ \space }
	}
}

\NewDocumentCommand \Xuline { m }
{
	\regex_split:nnN { \s } { #1 } \l_tmpa_seq
	\seq_map_indexed_inline:Nn \l_tmpa_seq
	{
		\underline{##2}
		\int_compare:nT { ##1 < \seq_count:N \l_tmpa_seq }
		{ \space }
	}
}


\RenewDocumentEnvironment { quote } { s O{1} }
{
	\str_case:nn { #2 }
	{
		{ 1 } { \dim_set:Nn \l_tmpa_dim { 1.35em} }
		{ 2 } { \dim_set:Nn \l_tmpa_dim { 3em} }
		{ 3 } { \dim_set:Nn \l_tmpa_dim { 4.5em } }
	}

	\IfBooleanF { #1 } {\Xbigskip}
	\begingroup
	\begin{adjustwidth}{\l_tmpa_dim}{1.34em}
}
{
	\end{adjustwidth}
	\endgroup
	\vspace{4pt}
	\IfBooleanF { #1 } {\Xbigskip}
}

\NewDocumentCommand \StartWongo { O{0} }
{
	\int_case:nnF { #1 }
	{
		{ 0 } { }
		{ 1 } { \Xnoindent\Xbigskip }
	}
	{
		\Xindent\Xnewline
		\int_step_inline:nn { #1 - 1 }
		{ \Xnoindent\Xbigskip }
	}
}

\NewDocumentCommand \Xminusbigskip {}
{
	\vspace*{-\baselineskip}
}

\NewDocumentCommand \Xbx { m }
{
	\str_set:Nn \l_tmpa_str { #1 }
	\regex_replace_all:nnN { (.)(.) } { \1\2 \| } \l_tmpa_str 
	\regex_split:nVN { \| } \l_tmpa_str \l_tmpa_seq
	\seq_remove_all:Nn \l_tmpa_seq {}
	\seq_map_inline:Nn \l_tmpa_seq
	{
		\X{##1}
	}
}

%\cs_new:Npn \Xbx_fn_noopt:n #1
%{
%	\group_begin:
%	\hbox_set:Nn \l_tmpa_box {	\spaceskip .2em plus .2em #1 }
%	\settowidth \l_tmpa_dim { \box_use:N \l_tmpa_box }
%	\dim_set:Nn \l_tmpa_dim { 1.34 \l_tmpa_dim }
%	\makebox [ \l_tmpa_dim ] [ s ] {	\spaceskip .2em plus .2em #1 }
%	\group_end:
%}

\NewDocumentCommand \Xnotopline {s}
{
	\IfBooleanF { #1 } {\clearpage}
	\begin{tikzpicture}[remember~picture,overlay]
	\tznode [red] (-1.5em,.8\baselineskip) {\adjustbox{width=3.5em,height=1.2em}{>}}
	\end{tikzpicture}
	\Xnewline
	\vspace*{-\dimexpr2\baselineskip-.18em\relax}
}

\NewDocumentCommand \Xmargdiscont {}
{
	\begin{tikzpicture}[overlay]
	\tznode [red](0,.7em){\adjustbox{width=1.3em}{\checkmark}}[r]
	\end{tikzpicture}%
	\hspace{2em}%
%	{\color{red}\rlap{\:\adjustbox{width=1.5em,height=1.2em}{\checkmark}}}
%	\hspace{1.5em}
}

\NewDocumentCommand \Xmargcont {}
{
	\begin{tikzpicture}[overlay]
	\tznode [red](-.65em,1.1em){\adjustbox{rotate=90,width=1.8em}{)}}[r]
	\end{tikzpicture}%
	\hspace{2em}%
}

\let\-\Xnoindent

\NewDocumentCommand \Eword { m }
{
	\tl_set:Nn \l_tmpa_tl { #1 }
	\regex_replace_all:nnN { (.)(.) } { \1\2| } \l_tmpa_tl
	\seq_set_split:NnV \l_tmpa_seq { | } \l_tmpa_tl
	\seq_remove_all:Nn \l_tmpa_seq {}
	\seq_map_inline:Nn \l_tmpa_seq 
	{
		\makebox[1.5em][c]{ ##1 }
	}
}

\bool_if:NT \opt_shortperiod_bool
{
	\renewcommand*\Xprd{\rlap{\;。}\allowbreak\hskip 1.5em plus .5pt minus 2pt}
	\dim_set:Nn \afterPrd_dim { 2.4em }
}

\ExplSyntaxOff

\let\*\Xbigskip

\let\XXtextbullet\textbullet
\renewcommand*\textbullet{{\hangulfontspec{Noto Serif CJK KR}\XXtextbullet}}

\newcommand*\itembullet{\noindent\llap{\X\textbullet }}
\newcommand*\itemenumopen{}
\newcommand*\itemenumclose{)}
\newlength{\itemenumafterspace}
\setlength{\itemenumafterspace}{0pt}
\newcommand*\itemenum{\refstepcounter{indentparenum}%
	\noindent
	\llap{\itemenumopen\X{\!\!\theindentparenum\hbox to0pt{\itemenumclose}}}%
	\hspace{\itemenumafterspace}%
}

\renewcommand\tableofcontents{%
	\begingroup
	\clubpenalty=0\widowpenalty=0\hyphenpenalty=0
	\fontspec{TeX Gyre Chorus}
	\hangulfontspec{UnPenheulim.ttf}[interhchar=0.08em,WordSpace={2,0,0}]
	\hanjafontspec{Noto Serif CJK KR}[Scale=.95,WordSpace={2,0,0}]
	\thispagestyle{wongoji}
    \@starttoc{toc}
	\endgroup
}

\maxtocdepth{subsection}
\maxsecnumdepth{subsection}

\endinput
